test_chain_spec

Function test_chain_spec 

pub fn test_chain_spec(fork: EthereumHardfork) -> Arc<ChainSpec> ⓘ
Returns the test chain spec with every hardfork up to and including fork active at genesis.

Panics

If fork can not be activated at genesis via ChainSpecBuilder.