E2ETestSetupExt

Trait E2ETestSetupExt 

pub trait E2ETestSetupExt: NodeBuilderHelper {
    // Provided methods
    fn test_setup(
        num_nodes: usize,
        chain_spec: Arc<Self::ChainSpec>,
    ) -> E2ETestSetupBuilder<Self> { ... }
    fn test_setup_for(fork: EthereumHardfork) -> E2ETestSetupBuilder<Self>
       where Self::ChainSpec: From<ChainSpec> { ... }
}
Extension trait to create an E2ETestSetupBuilder from a node type.

Provided Methods

fn test_setup( num_nodes: usize, chain_spec: Arc<Self::ChainSpec>, ) -> E2ETestSetupBuilder<Self>

Returns an E2ETestSetupBuilder for num_nodes nodes of this type.

fn test_setup_for(fork: EthereumHardfork) -> E2ETestSetupBuilder<Self>
where Self::ChainSpec: From<ChainSpec>,

Returns an E2ETestSetupBuilder for a single node of this type on the test_chain_spec with every hardfork up to and including fork active at genesis.

Use E2ETestSetupBuilder::with_num_nodes to launch more nodes.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors