pub fn test_signer(index: u32) -> PrivateKeySignerExpand description
Returns the signer of the test account at index, derived from TEST_MNEMONIC with the
default Ethereum derivation path m/44'/60'/0'/0/{index}.
pub fn test_signer(index: u32) -> PrivateKeySignerReturns the signer of the test account at index, derived from TEST_MNEMONIC with the
default Ethereum derivation path m/44'/60'/0'/0/{index}.