pub trait PendingTransactionExt<N: Network> {
// Required method
fn successful_receipt(
self,
) -> impl Future<Output = Result<N::ReceiptResponse>> + Send;
}Expand description
Extension trait for transactions sent through an alloy provider.
Required Methods
Sourcefn successful_receipt(
self,
) -> impl Future<Output = Result<N::ReceiptResponse>> + Send
fn successful_receipt( self, ) -> impl Future<Output = Result<N::ReceiptResponse>> + Send
Waits for the receipt of the transaction and returns it, failing if the transaction reverted.
Like [PendingTransactionBuilder::get_receipt], this waits until the transaction is
included, so the block containing it must be produced, e.g. by
NodeTestContext::advance_block or
dev mining.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".